// This is a dual-purpose class that controls the 
// PIC hardware. It can be used from an ordinary
// Java program or from JavaScript (via jioctl).
import javax.comm.*;
import java.io.*;

public class Jio
{
    private CommPortIdentifier cid; // Com port ID
    private SerialPort port;        // The actual port
    private InputStream is;         // input stream
    private OutputStream os;        // output stream 
    private boolean err;            // mark if an error occured
    public Jio() {}
    // open a port (name = COM1, /dev/xxx, etc.)
    public synchronized boolean openPort(String portname)
    {
        try {
            // open port (2 second time out)
            cid=CommPortIdentifier.getPortIdentifier(portname);
            port=(SerialPort)cid.open("JIOCTL",2000);
            // set parameters
            port.setSerialPortParams(19200,SerialPort.DATABITS_8,
                                     SerialPort.STOPBITS_1,
                                     SerialPort.PARITY_NONE);
            // no flow control
            port.setFlowControlMode(SerialPort.FLOWCONTROL_NONE);
            // get streams
            is=port.getInputStream();
            os=port.getOutputStream();
        }
        catch (Exception e) {
            err=true;
            return false;
        }
        err=false;
        return true;
    }

    // Set a pin high
    public synchronized void setHigh(int pin) 
    {
        try {
            os.write(0x10+pin);
        } catch (IOException e) { err=true; }
    }

    // Set a pin low
    public synchronized void setLow(int pin)
    {
        try {
            os.write(pin);
        } catch (IOException e) { err=true; }
    }

    // Toggle a pin high<->low
    public synchronized void toggle(int pin)
    {
        try {
            os.write(0x30+pin);
        } catch (IOException e) { err=true; }
    }

    // Read an input pin
    public synchronized boolean read(int pin)
    {
        int n=0;
        try {
            os.write(0x20+pin);
            n=is.read();
            System.err.println("Debug: n="+Integer.toString(n));
        } catch (IOException e) { err=true; }
        // convert from ASCII to boolean
        return n=='1';
    }

    // Close port
    public void closePort() throws IOException 
    {
        is.close();
        os.close();
        port.close();
    }

    // Handle the err variable for outsiders
    public boolean getErr() { return err; }
    public synchronized void setErr(boolean b) { err=b; }
    public void setErr() { setErr(false); }

}
