// This is the command interpreter that allows you
// to execute JavaScript with the Jio class that
// controls a PIC microcontroller 
// Al Williams http://www.al-williams.com

import org.mozilla.javascript.*;
import java.io.*;

// This object provides JavaScript services
// so it extends ScriptableObject 
public class Jioctl extends ScriptableObject {
    static Jioctl global;  // this object provides methods to the script
    // this is the main interpreter
    public static void main(String args[]) throws JavaScriptException
    {
        // set up script context
        Context cx=Context.enter();
        // extra things we provide
        String [] extras = { "print", "exit", "createJio" };
        global = new Jioctl();
        boolean interactive=false;
        try {
            // set up standard objects
            Scriptable scope=cx.initStandardObjects(global);
            // add our custom stuff
            global.defineFunctionProperties(extras,Jioctl.class,
                                            ScriptableObject.DONTENUM);
            String s,s1="";
            BufferedReader rdr=null;
            // if interactive then...
            if (args.length==0) 
                { // console
                    rdr=new BufferedReader(
                                           new InputStreamReader(System.in));
                    interactive=true;
                    // we do a separate loop for interactive
                    // entry so we can prompt and handle things line by line
                    while (true)
                        {
                            System.err.print(s1.length()==0?"IO> ":"- ");
                            s=rdr.readLine();
                            s1=s1+"\n"+s;
                            // look to see if we have a whole unit?
                            if (cx.stringIsCompilableUnit(s1))
                                {
                                    // if so run it!
                                    try {
                                    cx.evaluateString(
                                                                    scope,s1,
                                                                    "<cmd>",1,
                                                                    null);
                                    }
                                    catch (Exception e) {
                                        System.err.println(e.getMessage());
                                    }
                                    finally {
                                        s1="";
                                    }

                                }

                        }
                }
            else
                {  // not interactive
                    // in this case, we just let Rhino loose on the reader
                    try {
                        rdr=new BufferedReader(new FileReader(args[0]));
                    } catch (FileNotFoundException e) {
                            System.err.println("File not found");
                            System.exit(255);
                    }
                }
                cx.evaluateReader(scope,rdr,args[0],1,null);
        } catch (IOException e) {
            System.err.println(e.getMessage());
            System.exit(254);
        }
        catch (PropertyException e) {
            System.err.println(e.getMessage());
            System.exit(253);
        }
        finally {
            Context.exit();
        }
    }

    // script function: print
    public static void print(Context cx, Scriptable thisObj,
                             Object [] args, Function funObj)
    {
        for (int i=0;i<args.length;i++) {
            if (i>0) System.out.print(" ");
            String s = Context.toString(args[i]);
            System.out.print(s);
        }
        System.out.println();
    }

    // script function: exit (with exit code)
    public static void exit(Context cx, Scriptable thisObj,
                            Object[] args,Function funObj)
    {
        int rc=0;
        // take first argument if available
        if (args.length!=0)
            {
                String s=Context.toString(args[0]);
                rc=Integer.parseInt(s);
            }
        System.exit(rc);
    }

    // script function: Create a Jio!
    public static Object createJio()
    {
        Jio obj = new Jio();
        return (Object) obj;
    }

    // required by Scriptable Objects
    public String getClassName() { return "global"; }

}






