package com.mindsnack.mail;

import java.util.Properties;
import javax.mail.Message;
import javax.mail.MessagingException;
import javax.mail.Session;
import javax.mail.Transport;
import javax.mail.internet.AddressException;
import javax.mail.internet.InternetAddress;
import javax.mail.internet.MimeMessage;

import com.mindsnack.transactions.TransactionListener;
import com.mindsnack.transactions.Transaction;

public class MailSenderBean
  implements MailSender, TransactionListener
{
  MailMessageData data;
  String smtpServer;
  boolean inTransaction = false;
  boolean needToSend = false;

  // no arg constructor
  public MailSenderBean() {  }

  // a copy constructor
  public MailSenderBean(MailSenderBean source)
  {
    MailSenderBeanCopier copier = new MailSenderBeanCopier();
    copier.copy(source, this);
  }

  // usual constructor.
  public MailSenderBean(MailMessageData data, String smtpServer)
  {
    this.data = data;
    this.smtpServer = smtpServer;
  }

  public void setMailMessageData(MailMessageData data) { this.data = data; }
  public MailMessageData getMailMessageData() { return data; }

  public void setSmtpServer(String server) { this.smtpServer = server; }
  public String getSmtpServer() { return smtpServer; }

  public void send()
  {
    if (inTransaction) needToSend = true;
    else actualSend();
  }

  private void actualSend()
  {
    needToSend = false;
    try {
      Properties props = System.getProperties();
      props.put("mail.smtp.host", smtpServer);
      Session session = Session.getDefaultInstance(props, null);
      MimeMessage message = new MimeMessage(session);

      message.setFrom(new InternetAddress(data.getFrom()));
      message.addRecipient(Message.RecipientType.TO, new InternetAddress(data.getTo()));
      message.setSubject(data.getSubject());
      message.setText(data.getBody());

      Transport.send(message);
      numMailsActuallySent++;
    }
    catch (AddressException addressException) { System.out.println(addressException.getMessage()); }
    catch (MessagingException messagingException) { System.out.println(messagingException.getMessage()); }
  }

  int numMailsActuallySent = 0;  // For debugging.
  public int getNumMailsActuallySent() { return numMailsActuallySent; }

  public void begin(Transaction t) { inTransaction = true; }
  public boolean agreeToCommit(Transaction t) { return (data != null) && data.isValid(); }
  public void rollback(Transaction t) { inTransaction = false; needToSend = false; }
  public void commit(Transaction t) { inTransaction = false; if (needToSend == true) actualSend();}
}
