package com.mindsnack.test;

import java.io.InputStream;
import java.util.Date;
import java.util.Properties;

import junit.framework.TestCase;

import com.mindsnack.transactions.Copier;
import com.mindsnack.transactions.PlainCopier;
import com.mindsnack.transactions.SimpleTransaction;
import com.mindsnack.transactions.Transaction;

import com.mindsnack.mail.*;
import com.mindsnack.order.*;
import com.mindsnack.triangles.*;


public class TestCombined extends TestCase
{
  public TestCombined(String name)
    { super(name); }

  public void testTransactionCommit()
  {
    Transaction transaction = new SimpleTransaction();
    Copier orderCopier = new OrderCopier();
    Copier triangleCopier = new TriangleCopier();
    Copier mailCopier = new MailSenderBeanCopier();

    Order order = (Order) transaction.addToTransaction( new OrderImpl(), orderCopier );
    Triangle triangle = new TriangleImpl( 60.0, 60.0, 60.0 );
    triangle = (Triangle) transaction.addToTransaction(triangle, triangleCopier);
    MailSender mail = (MailSender) transaction.addToTransaction(getMailSender(), mailCopier);

    transaction.begin();
    {
      order.addItem("Pizza");
      triangle.setAngle(1,40.0);
      triangle.setAngle(2,80.0);  // both changes together should be OK.
      mail.send();
    }
    boolean result = transaction.commit();
    assertTrue(result);

    // Check that everything committed.
    assertTrue(triangle.getAngle(1) == 40.0);
    assertTrue(order.getNumItems()  == 1 );
    assertTrue(mail.getNumMailsActuallySent() ==  1);
  }

  public void testTransactionRollback()
  {
    Transaction transaction = new SimpleTransaction();
    Copier orderCopier = new OrderCopier();
    Copier triangleCopier = new TriangleCopier();
    Copier mailCopier = new MailSenderBeanCopier();

    Order order = (Order) transaction.addToTransaction( new OrderImpl(), orderCopier );
    Triangle triangle = new TriangleImpl( 60.0, 60.0, 60.0 );
    triangle = (Triangle) transaction.addToTransaction(triangle, triangleCopier);
    MailSender mail = (MailSender) transaction.addToTransaction(getMailSender(), mailCopier);

    transaction.begin();
    {
      order.addItem("Pizza");
      triangle.setAngle(1,40.0);  // should fail validity

      mail.send();
    }
    boolean result = transaction.commit();
    assertFalse(result);

    // Check that everything is rolled back.
    assertTrue(triangle.getAngle(1) == 60.0);
    assertTrue(order.getNumItems()  == 0 );
    assertTrue(mail.getNumMailsActuallySent() == 0);
  }



  /* Mail code */
  static Properties properties = new Properties();
  static int counter = 1;
  final static String MAIL_PROPERTIES_FILE = "mail.properties";
  static {
    InputStream is= TestMail.class.getResourceAsStream(MAIL_PROPERTIES_FILE);
    if (is != null)
      try { properties.load(is); } catch (Exception ignore) {}
  }

  private MailSender getMailSender()
  {
    MailMessageData mail = new MailMessageData();
    mail.setFrom( properties.getProperty("mail.from") );
    mail.setTo( properties.getProperty("mail.to") );
    mail.setSubject("Hello " + new Date());
    mail.setBody("Hello");
    String smtpServer = properties.getProperty("mail.smtp.server");
    MailSender mailSender = new MailSenderBean(mail, smtpServer );
    return mailSender;
  }
}
