package com.mindsnack.transactions;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationHandler;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

class TransactionProxy
  implements InvocationHandler
{

  Object original;
  Object duplicate;
  Copier copier;
  Transaction transaction;

  TransactionProxy
  (
    Object original,
    Copier copier,
    Transaction transaction
  )
    throws IllegalAccessException, InstantiationException
  {
    this.original = original;
    this.copier = copier;

    Class objectClass = original.getClass();
    Constructor copyConstructor = getCopyConstructor(objectClass);
    if (copyConstructor != null)
    {
      try {
        duplicate = copyConstructor.newInstance(new Object[] { original } );
      } catch (InvocationTargetException e) {}
    }

    if (duplicate == null)
    {
      duplicate = objectClass.newInstance();  // FUNKY...
      copier.copy(original, duplicate);
    }
    this.transaction = transaction;
  }

  private Constructor getCopyConstructor(Class objectClass)
  {
    Constructor copyConstructor = null;
    try {
      copyConstructor = objectClass.getConstructor( new Class[] { objectClass } );
    } catch (NoSuchMethodException e) {}

    return copyConstructor;
  }

  /**
   * @see InvocationHandler#invoke(Object, Method, Object[])
   */
  public Object invoke(Object object, Method method, Object[] args)
    throws Throwable
  {
    if (transaction.isBegun())
    {
      try {
        return method.invoke(duplicate, args);
      } catch (InvocationTargetException e)
        { throw e.getCause(); }
    } else {  /* not in transaction */
      String methodName = method.getName();
      if (methodName.startsWith("get") || methodName.startsWith("is"))
      {
        // These are presumed to be passive methods
        // that can be safely invoked outside of a transaction.
        try {
          return method.invoke(original, args);
        } catch (InvocationTargetException e)
          { throw e.getCause(); }
      } else
        throw new TransactionNotBegunException();
    }
  }
}
