package com.ericgiguere; // Always define a package!

import java.awt.*;
import java.awt.event.*;
import javax.microedition.xlet.*;

/**
 * A basic Xlet that puts up a single component and
 * waits for a keypress to terminate it. Use as the basis for
 * your own Xlets.
 */

public class BasicXlet implements Xlet {
    private XletContext     context;
    private Container       rootContainer;
    private Frame           rootFrame;
    private SimpleTextLabel label;

    public static void main( String args[] ){
        new XletRunner().runXlet( BasicXlet.class, args );
    }

    // Defer most initialization to the initXlet method

    public BasicXlet(){
    }

    // Called by the system to notify you that the
    // Xlet is about to be destroyed.

    public void destroyXlet( boolean unconditional )
                        throws XletStateChangeException {
        exit();
    }

    // Called by the Xlet to terminate itself.

    public void exit(){
        rootContainer.setVisible( false );
        context.notifyDestroyed();
    }

    // Returns the XletContext

    public XletContext getContext(){
        return context;
    }

    // Returns the root container

    public Container getRootContainer(){
        return rootContainer;
    }

    // Returns the root frame. The spec guarantees that
    // the root container is either a frame or has a
    // frame as an ancestor.

    public Frame getRootFrame(){
        return rootFrame;
    }

    // Obtains and stores the root UI components.

    private void initUI() throws XletStateChangeException {
        // Store the root container and traverse the
        // containment hierarchy to find the container's frame

        try {
            rootContainer = context.getContainer();

            Container tmp = rootContainer;
            while( !( tmp instanceof Frame ) ){
                tmp = tmp.getParent();
            }

            rootFrame = (Frame) tmp;
        }
        catch( UnavailableContainerException e ){
            // If we can't get the root container, abort
            // the initialization
            throw new XletStateChangeException( 
                "XletInitialization aborted -- no container: "
                 + e.getMessage() );
        }
        catch( NullPointerException e ){
            // This should never happen, but....
            throw new XletStateChangeException(
                "XletInitialization aborted -- no frame: "
                + e.getMessage() );
        }
    }

    // Called by the system to initialize the Xlet.

    public void initXlet( XletContext context )
                        throws XletStateChangeException {
        this.context = context; // store the context for later use
        initUI();

        // Now build our user interface

        label = new SimpleTextLabel( "Press a key to exit" );
        label.setBackground( Color.blue );
        label.setForeground( Color.yellow );
        label.addKeyListener( new KeyAdapter(){
            public void keyTyped( KeyEvent e ){
                exit();
            }
        } );

        rootContainer.add( label );

        // Don't make the container visible here in
        // case it's a frame, defer it until the
        // startXlet method is called.
    }

    // Called by the system to pause the Xlet.
    // This is where you release any system resources
    // you are using. The system is likely to make
    // the root frame invisible, so there's no reason
    // to make the container invisible.

    public void pauseXlet(){
    }

    // Called by the system to activate the Xlet.

    public void startXlet() throws XletStateChangeException {

        // Make sure the container is visible

        if( !rootContainer.isVisible() ){
            rootContainer.setVisible( true );
        }
    }
}
